// 'Implementations' of some tiny libc things
// Matt Evans 28Mar05

#include "asmfns.h"
#include "time.h"

// This is just shoved here for lack of a better (C) place to put it:

// Get a byte from serial:  returns -1 if timed out.
int uart_getbyte_timeout(int millisecs)
{
  unsigned int tFirst = time_now();
  
  millisecs *= TIME_TICKS_PER_MS;
  
  while (!uart_testrx())
  {
    if (time_now() > (tFirst + millisecs))
    {
      return -1;
    }
  }
  // Oh! A char is there:
  // do not sign extend, return just 8 bits straight:
  return (int)((unsigned int)((unsigned char)uart_getbyte()));
}

// Get a buffer load...
int uart_getbuffer_timeout(unsigned char *buffer, int len, int millisecs)
{
  int i;
  millisecs *= TIME_TICKS_PER_MS;
  
  for (i = 0 ; i < len; i++)
  {
    unsigned int tFirst = time_now();

    while (!uart_testrx())
    {
      if (time_now() > (tFirst + millisecs))
      {
        return -1;  // timed out!
      }
    }
    buffer[i] = uart_getbyte();
  }
  return 0;
}

// OK so this isn't libc.. :p
char *skipwhitespace(char *str)
{
  while ((*str != '\0') && ((*str == ' ') || (*str == '\t'))) { str++; }
  return str;
}

inline char tolower(char c)
{
  return c | 0x20;
}

// ascii to hex
// success = 0 if couldn't parse input (e.g. EOL, or not hex)
int atoh(char *c, char **end, int *success)
{
  int r = 0;
  int ok = 0;
  char digit = *c;
  char lowerd = tolower(digit);
  
  while (digit != 0 && ((digit >= '0' && digit <= '9') || (lowerd >= 'a' && lowerd <= 'f')))
  {
    int v;
    r = r << 4;
    v = (lowerd >= 'a') ? (lowerd - 'a' + 10) : (lowerd - '0');
    r |= v;
    c++;
    digit = *c;
    lowerd = tolower(digit);
    ok = 1;
  }
  if (ok)
  {
    *end = c;
  }
  *success = ok;
  return r;
}

// libc, where are you?
int strcmp(char *a, char *b)
{
  int end = 0;
  do
  {
    if (*a != *b)
    { 
      return 1;
    }
    if (*a == 0)
    {
      end = 1;
    }
  }
  while (!end);
  return 0;
}

int strncmp(char *a, char *b, int len)
{
  for (; len > 0; len--)
  {
    if ((*a != *b) || (*a == '\0') || (*b == '\0'))
    { 
      return 1;
    }
    a++;
    b++;
  }
  return 0;
}

void strcpy(char *dest, char *from)
{
  while (*from)
  {
    *dest++ = *from++;
  }
}

// The least optimal memcpy in the world:
void memcpy(char *dest, char *from, int num)
{
  for (; num > 0; num--)
  {
    *dest++ = *from++;
  }
}

void memset(char *dest, int val, int len)
{
  while (len-- > 0)
  {
    *dest++ = (char)val;
  }
}

// Dumb software timing loop. 
// It's not that accurate, just pass it a rough time...
void usleep(int usecs)
{
  int i, j;
  
  for (i = 0; i < usecs; i++)
  {
    for (j = 0; j < 8; j++)  // vaguely timed by hand..
    { ;
    }
  }
}

static unsigned int __randomseed = 0x5047;
unsigned int __reg1thing = 1;

// From ARM's code examples:
unsigned int random(void)
{
  asm("tst  %3, %3, lsr#1 \n"
      "movs r3, %0, rrx \n"
      "adc  %1, %3, %3 \n"
      "eor  r3, r3, %0, lsl#12 \n"
      "eor  %2, r3, r3, lsr#20 \n"
      : "=r"(__randomseed), "=r"(__reg1thing) : "r"(__randomseed), "r"(__reg1thing) : "r3");
  return __randomseed;
}
