/* Low-level NAND access for E2, via its CPLD.  Should eventually be wrapped
   in a nice abstraction 'block device' layer.

   me 26Mar05
*/

#ifndef _NAND_H
#define _NAND_H

struct nandInfo
{
  int chipsel;  // The CPLD control reg bits used to select this device.  0 means no device present!
  int manuf;
  int device;
  // The smallest-erasable chunk.  The 30lv008 has 512byte pages but 
  // is only erasable in 8K blocks.  Though, you can program individual pages it's
  // not always useful if you have to erase 8K adjacent...
  int blocksize;
  int pagesize;
  // Capacity in bytes:
  int capacity;
  
  // e.g. some NAND chips might not have same instruction set, might not do page reads, etc.  FIXME
  int capabilities; 
};
#define NAND_CAP_FOUR_BYTE_READ 1

#define MAX_FLASH_DEVS 2

struct nandManu
{
  int IDcode;
  char *name;
};

struct nandDevice
{
  int IDcode;
  int capacity;
  int pagesize;
  int blocksize;
};
// Errors returned by various fns:

#define NAND_EINVAL 10
#define NAND_ENODEV 11
#define NAND_EREADERROR 12
#define NAND_EWRITEERROR 13

void nand_init(void);
int nand_isExtCardPresent(void);
int nand_enumerate(struct nandInfo *inf, int *numfound);
int nand_read(struct nandInfo *dev, unsigned char *dest, unsigned int offset, unsigned int length);
int nand_write(struct nandInfo *dev, unsigned char *from, unsigned int dest_offset, unsigned int length);
void nand_listdevs(struct nandInfo *ni);
#endif
