// Support for calls into the PBL ROM's routines

// Matt Evans, April 2005
#include "pblcalls.h"


void pbl_generate_flashdevstruct(struct pblFlashDev *dev, int size, int pgsize, int blksize, int intExt)
{
  dev->size = size;
  dev->pgsize = pgsize;
  dev->pgsizeAgain = pgsize;
  dev->blksize = blksize;
  dev->intext = intExt; 
}

// Fn ptrs:

// Read data from the NAND flash (through the FTL-style stuff) into outbuffer.  Make sure
// offset and length are block-aligned...?  They're aligned to something, at least a page.
int (*pbl_readflash)(unsigned char *outbuffer, int offset, int length) = (int (*)())(PBL_BASE + PBL_READNAND);

// Erase a block in the flash:  blocks are 8K, so align the offset and length to this amount.
int (*pbl_eraseflash)(int offset, int length) = (int (*)())(PBL_BASE + PBL_ERASENAND);

// Set the current flash type before program:
void (*pbl_setfltype)(unsigned char type) = (void (*)())(PBL_BASE + PBL_SET_CURRENTFLASHTYPE);

// Program blocks:  Some state needs to be set up before the main routine is called.
// Again, align the offset and length to 8K (any less and some data won't get written).
int (*__pbl_int_pgmflash)(int strangeoffset, int offset, unsigned char *data, int length) = 
    (int (*)())(PBL_BASE + PBL_PROGNAND);

int pbl_programflash(int offset, unsigned char *data, int len)
{
  // Important:  In case PBL has left it in an inconsistent state (or just not initialised it)
  // If we set it to NAND then screwing up the NOR flash is less likely ;)
  pbl_setfltype(PBL_FLTYPE_NAND);
  
  return __pbl_int_pgmflash(0x80010000, offset, data, len);
}


// This is in PBL's data section.  We link/live much higher up so it is preserved:
struct pblFlashInfo **pbl_ptrToFlashInfo = (struct pblFlashInfo **)PBL_FLASHINFO;
