// Calls into PBL ROM, for v3.1, build 1277

// There are various low-level calls to read/write NAND flash blocks, but they're
// nicely wrapped up in the routines below which perform ECC and, I think, logical-to-physical
// translation.

// Matt Evans, April 2005

#ifndef _PBLCALLS_H
#define _PBLCALLS_H

// We call PBL in RAM, here:
#define PBL_BASE 0
// Address of read flash routine:
#define PBL_READNAND 0x4d18
// Address of erase flash routine:
#define PBL_ERASENAND 0x841c
// Addr of program flash routine:
#define PBL_PROGNAND 0x687c
// Set the global for current flash type to r0:
#define PBL_SET_CURRENTFLASHTYPE 0x686c

// There's mention of this address in PBL as the max size.  Hm.  5.5MB out of an 8MB device?
#define MAXFLASHADDR 0x550000

// Address of a struct pblFlashinfo *:
#define PBL_FLASHINFO 0x87fc
struct pblFlashDev
{
  int size;
  int pgsize;
  int blksize;
  int intext;
  int pgsizeAgain;
};

struct pblFlashInfo
{
  int valid;  // ?  seems to be checked for 0ness, is usually 1.
  int unkB;   // directory block offset?
  int unkC;
  int unkD;
  struct pblFlashDev fdev;
};

void pbl_generate_flashdevstruct(struct pblFlashDev *dev, int size, int pgsize, int blksize, int intExt);

// 0x4d18 (goes to 4d20 with r3 = 1)
extern int (*pbl_readflash)(unsigned char *outbuffer, int offset, int length);

extern int (*pbl_eraseflash)(int offset, int length);

#define PBL_FLTYPE_NAND 4
extern void (*pbl_setfltype)(unsigned char type);

int pbl_programflash(int offset, unsigned char *data, int len);
extern int (*__pbl_int_pgmflash)(int strangeoffset, int offset, unsigned char *data, int length);

extern struct pblFlashInfo **pbl_ptrToFlashInfo;

#endif
