// Noddy uart functions for the Amstrad E2/Sharp LH79531
// (c) Matt Evans 23/3/05

#include "peripherals.h"

.global uart_getbyte, uart_getbyte_nonblock, uart_putbyte, uart_putstring, uart_testrx, setup_uart
.global uart_phex8, uart_phex32, uart_pnewl, uart_putbuffer, uart_getbuffer

  // Blocking getbyte (i.e. waits until one ready)
uart_getbyte:
  mov     r0, #0x3f4000
  sub     r0, r0, #0x400000
1:
  ldrb    r1, [r0, #UART_FLG] // Flag register
  tst     r1, #0x10           // RX Fifo empty
  bne     1b
  ldrb    r0, [r0]
  mov     pc, lr

  // textrx: returns true if character waiting
uart_testrx:
  mov     r0, #0x3f4000
  sub     r0, r0, #0x400000
  ldrb    r0, [r0, #UART_FLG] // Flag register
  tst     r0, #0x10           // RX Fifo empty
  movne   r0, #0
  moveq   r0, #1
  mov     pc, lr
  
  // Returns r0 = byte received, with *param = 1 for success and = 0 for 'not ready yet'
uart_getbyte_nonblock:
  mov     r1, r0
  mov     r0, #0x3f4000
  sub     r0, r0, #0x400000
  ldrb    r2, [r0, #UART_FLG] // Flag register
  tst     r2, #0x10           // RX Fifo empty
  movne   r2, #0
  moveq   r2, #1              // success!
  str     r2, [r1]
  ldreqb  r0, [r0]
  mov     pc, lr
    
  // R0 = byte to put
uart_putbyte:
  mov     r1, #0x3f4000
  sub     r1, r1, #0x400000
1:
  ldrb    r2, [r1, #UART_FLG]
  tst     r2, #0x20           // TXFifo full
  bne     1b
  strb    r0, [r1]
  mov     pc, lr
  
uart_putstring:
  // R0 = string ptr.
  mov     r3, #0x3f4000
  sub     r3, r3, #0x400000
1:
  ldrb    r1,[r0],#1
  cmp     r1,#0
  moveq   pc, r14
3:
  ldrb    r2, [r3, #UART_FLG]       // tx status
  tst     r2, #0x20           // txfifo full?
  bne     3b
    
  strb    r1, [r3]
  b       1b

uart_phex8:
  stmfd   r13!,{r4, r14}
  // R0 = byte to print
  mov     r4,#2
  mov     r3, r0, lsl#24
  b       uart_phex_gen

uart_phex32:
  stmfd   r13!,{r4, r14}
  // R0 = word to print
  mov     r4,#8
  mov     r3, r0
uart_phex_gen:
  mov     r0, r3, lsr#28
  cmp     r0, #10
  addge   r0, r0, #'a'-10
  addlt   r0, r0, #'0'
  bl      uart_putbyte
  mov     r3, r3, lsl#4
  subs    r4, r4, #1
  bgt     uart_phex_gen
  
  ldmfd   r13!,{r4, pc}
  
uart_pnewl:
  stmfd   r13!, {r14}
  mov     r0, #'\r'
  bl      uart_putbyte
  mov     r0, #'\n'
  bl      uart_putbyte
  ldmfd   r13!, {pc}
  
  
// UART to 115200, 8N1
setup_uart:
  // Setup a clock based off SYSCLK (Should still be running off PLL?)
  mov     r3, #0
  sub     r3, r3, #0x8800       // r3 = ffff7800 (RPC base)
  ldr     r0, [r3, #RPC_MACROCLKSEL]
  bic     r0, r0, #0x1c         // UART 0, 1, 2 from system clock
  str     r0, [r3, #RPC_MACROCLKSEL]
  ldr     r0, [r3, #RPC_MACROCLKCTL]
  bic     r0, r0, #1<<10        // UART 0 clock definitely not switched off now..
  str     r0, [r3, #RPC_MACROCLKCTL]

  mov     r1, #0  
  sub     r1, r1, #0xc000     // r1 = ffff4000 (UART0)
  add     r3, r1, #0xc00      // r3 = ffff4c00 (PIO base)
  // Setup pins MUX to UART not GPIO:
  ldr     r0, [r3, #PIO_FMUXCTRL1]
  bic     r0, r0, #0xc0
  str     r0, [r3, #PIO_FMUXCTRL1]
  ldr     r0, [r3, #PIO_FMUXCTRL2]
  bic     r0, r0, #0xc0
  str     r0, [r3, #PIO_FMUXCTRL2]

  // Configure UART0 to 115200 8N1
  // 54 => For 57600:
  // mov     r0, #54            
  // 27/26 (26!) => For 1152000:
  mov     r0, #26
  // For 9600:
  /* mov     r0, #0x100
     orr     r0, r0, #0x045 
     */
  mov     r2, #0
  strb    r2, [r1, #UART_CON]
  strb    r0, [r1, #UART_BR_LCRL]
  mov     r0, r0, lsr #8
  strb    r0, [r1, #UART_BR_LCRM]
  mov     r0, #0x60               // FIFO, 8N1 (we want fifo..?  off. 0x70 with on.)
  strb    r0, [r1, #UART_BR_LCRH]
  // should probably drain FIFOs?
  mov     r0, #0xf
  strb    r0, [r1, #UART_INTMSK]  // All IRQs masked
  mov     r0, #1
  strb    r0, [r1, #UART_CON]     // UART_EN
  mov     pc, lr

uart_putbuffer:
  // r0 = destination buffer, r1 = count
  stmfd   r13!, {r4}
  mov     r3, #0x3f4000
  sub     r3, r3, #0x400000
1:
  ldrb    r4,[r0],#1
3:
  ldrb    r2, [r3, #UART_FLG]     // tx status
  tst     r2, #0x20               // txfifo full?
  bne     3b
    
  strb    r4, [r3]
  subs    r1, r1, #1
  bgt     1b

  ldmfd   r13!, {r4}
  mov     pc, r14
    
uart_getbuffer:
  // r0 = destination buffer, r1 = count
  mov     r2, #0x3f4000
  sub     r2, r2, #0x400000
1:
  ldrb    r3, [r2, #UART_FLG]     // Flag register
  tst     r3, #0x10               // RX Fifo empty
  bne     1b

  ldrb    r3, [r2]
  strb    r3, [r0], #1
  subs    r1, r1, #1
  bgt     1b
  mov     pc, lr

// End
