/* (c) Matt Evans 10 April 2005
   xmodem upload/download code
   Massive to-do in here:  The initial handshaking is very broken & will only work on (so far) ZTerm.
   It needs to be made more robust, including sending timeouts when it's upset that nothing's arrived.
*/

#include "xmodem.h"
#include "asmfns.h"
#include "crc.h"
#include "libcfns.h"
#include "cons.h"

//#define XDEBUG

int usingCRC;

void xmodem_init(void)
{
  
}

static int xmodem_csum(unsigned char *buffer, int len)
{
  int csum = 0;
  int i;
  
  for (i = 0; i < len; i++)
  {
    csum = (csum + buffer[i]) & 0xff;
  }
  return csum;
}

void xmodem_send(unsigned char *buffer, int len)
{
  struct pkt128 yp;
  struct pkt1K ypL;   // large packet...
  unsigned short crc;
  int usingCRC;
  int seq = 0;
  int c, c1;
  int blocksToSend = ((len+1023) & ~1023) >> 10;  // Number of 1K blocks, rounded up
//  int lastBlockLen = ((len & 1023) == 0) ? 1024 : (len & 1023);
  int block;
  int retries;
  
  uart_putstring("Start your XMODEM download now\r\n");

  c1 = uart_getbyte_timeout(60000);
  
  if (c1 == -1)
  {
    uart_putstring("Timeout waiting for receiver\r\n");
    return;
  }
  else if (c1 != 'C' && c1 != NAK)
  {
    uart_putstring("Receiver sent weird hello: ");
    uart_phex8(c1);
    uart_pnewl();
    return;
  }
  
  if (c1 == 'C')
  {
    usingCRC = 1;
  }
  else
  {
    usingCRC = 0;
  }
  
  // Just send a bunch of 1024byte blocks; the last one can be 128 for optimal transmission
  // but... for the moment I'm lazy.
  for (block = 0; block < blocksToSend; block++)
  {
    int blockLen = 1024;  // Even if not sending short last block, this should be 'handshook' upon
    seq++;
    
    /// Drain input buffer here...
    
    if (blockLen > 128)
    {
      ypL.soh = STX;  // 1024-byte block
      ypL.seq1 = seq;
      ypL.seq2 = ~seq;
      memcpy(ypL.data, &buffer[block*1024], blockLen);
      
      if (usingCRC)
      {
        ypL.crc1 = 0;
        ypL.crc2 = 0;
        crc = crc16_buf(ypL.data, 1024+2);
        ypL.crc1 = ((crc >> 8) & 0xff);
        ypL.crc2 = (crc & 0xff);
      }
      else
      {
        ypL.crc1 = xmodem_csum(ypL.data, blockLen);
      }
    }
    else
    {
      yp.soh = SOH; // 128-byte block
      yp.seq1 = seq;
      yp.seq2 = ~seq;
      memcpy(yp.data, &buffer[block*128], blockLen);

      if (usingCRC)
      {
        yp.crc1 = 0;
        yp.crc2 = 0;
        crc = crc16_buf(yp.data, 128+2); 
        yp.crc1 = ((crc >> 8) & 0xff);
        yp.crc2 = (crc & 0xff);
      }
      else
      {
        yp.crc1 = xmodem_csum(yp.data, blockLen);
      }      
    }

    // Don't yet pad the buffer with EOFs for the last block.... FIXME
    retries = 5;
    do 
    {
      if (blockLen > 128)
      {
        uart_putbuffer((unsigned char *)&ypL, (usingCRC ? SIZEOF_1K : SIZEOF_1K-1));
      }
      else
      {
        uart_putbuffer((unsigned char *)&yp, (usingCRC ? SIZEOF_128 : SIZEOF_128-1));
      }
      // Get ACK/NAK:
      c = uart_getbyte_timeout(1000);
      if (c == -1)
      {
        if (retries-- > 0)
        {
          c = NAK; // just cause a resend
        }
        else
        {
          uart_putstring("Timeout on sending block!\r\n");
          return;
        }
      }
    } while (c == NAK);  // NAK, resend.
    if (c != ACK)
    {
      // Was it something else?
      uart_putstring("Aborting transfer, invalid char ");
      uart_phex8(c);
      uart_pnewl();
      return;
    }
  }
  do 
  {
    uart_putbyte(EOT);
    // Sends a NAK here, send another EOT then ACK?
    c = uart_getbyte_timeout(1000);
  } while ((c != -1) && (c != ACK));
  if (c == -1)
  {
    uart_putstring("Timeout on final ACK\r\n");
  }
}

void xmodem_receive(unsigned char *buffer)
{
  int usingCRC;
  int rxSoFar = 0;
  int done;
  int tries;
  int c;
  int firstGo = 1;
  unsigned char seq = 1;
  
  // Send a 'C' to start off
  done = 0;
  tries = 25;

  do
  {
    uart_putbyte('C');
    
    c = uart_getbyte_timeout(1000);
    if (c == -1)
    { // timeout
      // Still haven't heard from the other end; so send another 'C'
      if (tries-- < 0)
      {
        done = 1;
      }
    }
    else
    {
      done = 1;
    }
    // Should really keep retrying until we get a block through - sender might not support CRC, too
    // We assume they MUST support CRC - we don't support checksum yet. 
  } while (!done);
  
  if (tries < 0)
  {
    uart_putstring("Receive timed out waiting for sender\r\n");
    return;
  }
  
/*  if (c == SOH || c == STX)  // or stx..
  {
    // blah, this means something... buffer size or something?
    uart_putbyte(NAK); //ACK);  // WHY???
  }
  else
  {
    uart_putstring("Receive: weird hello from sender ");
    uart_phex8(c);
    uart_pnewl();
    return;
  }    
  */
  usingCRC = 1;
  tries = 10;
  done = 0;
  do
  {
    int size = 0;
    
    if (firstGo)
    {
      // Urgh. Already got the byte, above.
      firstGo = 0;
    }
    else
    {
      c = uart_getbyte_timeout(2000);
    }
#ifdef XDEBUG
    cons_putbyte('[');
    cons_phex8(c);
#endif
    
    switch (c)
    {
      case SOH:  // 128
        size = 128;
        break;
        
      case STX:  // 1024
        size = 1024;
        break;
        
      case EOT:  // Sender's finished.
        uart_putbyte(ACK);
        done = 1;
        break;

      case -1:
        // timeout
        if (tries-- < 0)
        {
          done = 1;
        }
        break;
        
      case CAN:
        uart_putbyte(NAK);
        continue;
     
      default:
        cons_putstring("Aborting receive, weird char in header: ");
        cons_phex8(c);
        cons_putbyte('\n');
//        done = 1;
        goto abort_xfer;  // ew, I'm sorry...
        break;
    }
    
    if (!done)
    {
      int seq1 = 0, seq2 = 0;
      int crc1, crc2;
      int blockOK = 1;
      unsigned char rxbuff[1024+2];

#ifdef XDEBUG
      cons_putbyte(':');
#endif
      seq1 = uart_getbyte_timeout(500);
#ifdef XDEBUG
      cons_phex8(seq1);
#endif
      if (seq1 >= 0)
      {
        seq2 = uart_getbyte_timeout(500);
#ifdef XDEBUG
        cons_phex8(seq2);
        cons_putbyte(':');
#endif
      }
      
      if ((seq1 == -1) || (seq2 == -1) || 
          ((seq1 & 0xff) != (~seq2 & 0xff)) || (seq1 != seq)) 
      {
        // This is just plain bad:
//        blockOK = 0; // send a NAK on timeout
        cons_putstring("Bad sequence number ");
        cons_phex8(seq1);
        cons_phex8(seq2);
        cons_putstring("! (Current ");
        cons_phex8(seq);
        cons_putbyte(')');
        cons_putbyte('\n');
        
        goto abort_xfer;  // again, ew
      }
      else
      {
        int tout;
        
        seq++;
        // receive block
        tout = uart_getbuffer_timeout(rxbuff, size, 5000);
        
#ifdef XDEBUG
        cons_phex8(tout);
        cons_putbyte(';');
#endif
        if (tout == -1)
        {
          blockOK = 0;
        }
        else
        {
          crc1 = uart_getbyte_timeout(1000);
#ifdef XDEBUG
          cons_putbyte('-');
          cons_phex8(crc1);
#endif
          if (crc1 == -1)
          {
            // timeout
            blockOK = 0;
          }
          else
          {
            if (usingCRC)
            {
              unsigned short s;
              crc2 = uart_getbyte_timeout(1000);
#ifdef XDEBUG
              cons_phex8(crc2);
#endif
              if (crc2 == -1)
              {
                blockOK = 0;
              }
              else
              {
                rxbuff[size] = 0;
                rxbuff[size+1] = 0;
                s = crc16_buf(rxbuff, size+2);
                
                if ((crc1 != ((s >> 8) & 0xff)) || (crc2 != (s & 0xff)))
                {
#ifdef XDEBUG
                  cons_putbyte('c');
                  cons_phex32(s);
                  cons_putbyte('!');
#endif
                  blockOK = 0;
                }
              }
            }
            else
            {
              int i;
              int sum = 0;
              for (i = 0; i < size; i++)
              {
                sum = (sum + rxbuff[i]) & 0xff;
              }
              if (crc1 != sum)
              {
                blockOK = 0;
              }
            }
          }
        }
      }
      
      if (blockOK)
      {
        memcpy(&buffer[rxSoFar], rxbuff, size);
        rxSoFar += size;
        uart_putbyte(ACK);
#ifdef XDEBUG
        cons_putbyte('Y');
#endif
      }
      else
      {
        uart_putbyte(NAK);
#ifdef XDEBUG
        cons_putstring("N!!!    ");
#endif
      }
#ifdef XDEBUG
      cons_putbyte(']');
#endif
    }
    else
    {
      uart_putstring("Transfer finished; received ");
      uart_phex32(rxSoFar);
      uart_putstring(" bytes.\r\n");
    }
  } while (!done);
  
  return;
  
abort_xfer:
    uart_putbyte(CAN);        uart_putbyte(CAN);
    uart_putbyte(CAN);        uart_putbyte(CAN);
    uart_putbyte(CAN);        uart_putbyte(CAN);
    uart_putbyte(CAN);        uart_putbyte(CAN);
    uart_putstring("Uhoh!  Transfer aborted!\r\n");
    
    return;
}
